import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;
import javax.swing.event.*;

public class MenuCreator extends JPanel implements ActionListener, CaretListener, ListSelectionListener {

  private BufferedImage bimgLeftArrow;
  private BufferedImage bimgRightArrow;
  private BufferedImage bimgUpArrow;
  private BufferedImage bimgDownArrow;

  private JTextField tfCaption;
  private JTextField tfName;
 
  private JButton btnOk, btnCancel;
  private JButton btnLeft, btnRight, btnUp, btnDown;
  private JButton btnNext, btnInsert, btnDelete;

  private JCheckBox chbEnabled, chbVisible;

  private JList lstMenuEntries;
  private DefaultListModel dlmMenuEntries;
  
  private JScrollPane spMenuEntries;
  private JViewport vpMenuEntries;

  private JButton btnClipboard;
  
  private JComboBox cbItemtype;
  private JCheckBox chbChecked;
  private JComboBox cbOptionGroup;
  private JComboBox cbShortcut;
  private JLabel lblOptionGroup;

  private final String[] shortcut = {"(None)", "Ctrl+A", "Ctrl+B", "Ctrl+C", "Ctrl+D", "Ctrl+E", "Ctrl+F",
      "Ctrl+G", "Ctrl+H", "Ctrl+I", "Ctrl+J", "Ctrl+K", "Ctrl+L", "Ctrl+M", "Ctrl+N", "Ctrl+O", "Ctrl+P",
      "Ctrl+Q", "Ctrl+R", "Ctrl+S", "Ctrl+T", "Ctrl+U", "Ctrl+V", "Ctrl+W", "Ctrl+X", "Ctrl+Y", "Ctrl+Z",
      "F1", "F2", "F3", "F4", "F5", "F6", "F7", "F8", "F9", "F10", "F11", "F12",
      "Ctrl+F1", "Ctrl+F2", "Ctrl+F3", "Ctrl+F4", "Ctrl+F5", "Ctrl+F6", "Ctrl+F7", "Ctrl+F8", "Ctrl+F9",
      "Ctrl+F10", "Ctrl+F11", "Ctrl+F12",
      "Shift+Ctrl+F1", "Shift+Ctrl+F2", "Shift+Ctrl+F3", "Shift+Ctrl+F4", "Shift+Ctrl+F5", "Shift+Ctrl+F6",
      "Shift+Ctrl+F7", "Shift+Ctrl+F8", "Shift+Ctrl+F9", "Shift+Ctrl+F10", "Shift+Ctrl+F11", "Shift+Ctrl+F12",
      "Ctrl+Ins", "Shift+Ins", "Del", "Shift+Del", "Alt+Bksp"};
  
  private final String[] itemtype = {"Standard", "CheckBox", "OptionButton"};
  private final String[] optiongroup = {"0", "1", "2", "3", "4", "5", "6", "7", "8", "9"};
  

  private MenuEntriesData[] menuEntriesData;

  /* constructor */
  public MenuCreator() {
    setLayout(null);
    setPreferredSize(new Dimension(520,480));

    JLabel lblCaption = new JLabel("Caption:");
    lblCaption.setBounds(15,18,70,25);
    add(lblCaption);

    tfCaption = new JTextField();
    tfCaption.addCaretListener(this);
    tfCaption.setBounds(90,18,200,25);
    JList l = new JList();
    tfCaption.setBackground(l.getBackground());
    tfCaption.setForeground(l.getForeground());
    add(tfCaption);

    JLabel lblName = new JLabel("Name:");
    lblName.setBounds(15,50,70,25);
    add(lblName);

    tfName = new JTextField();
    tfName.addCaretListener(this);
    tfName.setBounds(90,50,200,25);
    tfName.setBackground(l.getBackground());
    tfName.setForeground(l.getForeground());
    add(tfName);
    
    JLabel lblShortcut = new JLabel("Shortcut:");
    lblShortcut.setBounds(15,82,70,25);
    add(lblShortcut);

    cbShortcut = new JComboBox(shortcut);
    cbShortcut.addActionListener(this);
    cbShortcut.setBounds(90,82,200,25);
    cbShortcut.setBackground(l.getBackground());
    cbShortcut.setForeground(l.getForeground());
    add(cbShortcut);

    
    JLabel lblItemType = new JLabel("ItemType:");
    lblItemType.setBounds(15,118,70,25);
    add(lblItemType);

    cbItemtype = new JComboBox(itemtype);
    cbItemtype.addActionListener(this);
    cbItemtype.setBounds(90,118,150,25);
    cbItemtype.setBackground(l.getBackground());
    cbItemtype.setForeground(l.getForeground());
    add(cbItemtype);

    chbChecked = new JCheckBox("Checked");
    chbChecked.setSelected(true);
    chbChecked.addActionListener(this);
    chbChecked.setMnemonic('h');
    chbChecked.setBounds(250,118,100,25);
    chbChecked.setVisible(false);
    add(chbChecked);

    lblOptionGroup = new JLabel("OptionGroup:");
    lblOptionGroup.setBounds(350,118,100,25);
    lblOptionGroup.setVisible(false);
    add(lblOptionGroup);

    cbOptionGroup = new JComboBox(optiongroup);
    cbOptionGroup.addActionListener(this);
    cbOptionGroup.setBounds(450,118,50,25);
    cbOptionGroup.setBackground(l.getBackground());
    cbOptionGroup.setForeground(l.getForeground());
    cbOptionGroup.setVisible(false);
    add(cbOptionGroup);
    
    chbEnabled = new JCheckBox("Enabled");
    chbEnabled.setSelected(true);
    chbEnabled.addActionListener(this);
    chbEnabled.setMnemonic('E');
    chbEnabled.setBounds(15,150,105,25);
    add(chbEnabled);

    chbVisible = new JCheckBox("Visible");
    chbVisible.setSelected(true);
    chbVisible.addActionListener(this);
    chbVisible.setMnemonic('V');
    chbVisible.setBounds(127,150,105,25);
    add(chbVisible);

    bimgLeftArrow = new BufferedImage(9, 9, 2);
    bimgRightArrow = new BufferedImage(9, 9, 2);
    bimgUpArrow = new BufferedImage(9, 9, 2);
    bimgDownArrow = new BufferedImage(9, 9, 2);
    createButtonImages();


    btnLeft = new JButton(new ImageIcon(bimgLeftArrow));
    btnLeft.setBounds(39,186,20,23);
    btnLeft.addActionListener(this);
    add(btnLeft);

    btnRight = new JButton(new ImageIcon(bimgRightArrow));
    btnRight.setBounds(63,186,20,23);
    btnRight.addActionListener(this);
    add(btnRight);

    btnUp = new JButton(new ImageIcon(bimgUpArrow));
    btnUp.setBounds(87,186,20,23);
    btnUp.addActionListener(this);
    add(btnUp);

    btnDown = new JButton(new ImageIcon(bimgDownArrow));
    btnDown.setBounds(111,186,20,23);
    btnDown.addActionListener(this);
    add(btnDown);

    btnNext = new JButton("Next");
    btnNext.setBounds(151,186,89,23);
    btnNext.setMnemonic('N');
    btnNext.addActionListener(this);
    add(btnNext);

    btnInsert = new JButton("Insert");
    btnInsert.setBounds(247,186,89,23);
    btnInsert.setMnemonic('I');
    btnInsert.addActionListener(this);
    add(btnInsert);

    btnDelete = new JButton("Delete");
    btnDelete.setBounds(343,186,89,23);
    btnDelete.setMnemonic('T');
    btnDelete.addActionListener(this);
    add(btnDelete);

    menuEntriesData = new MenuEntriesData[5000];  // For performance reasons a constant array size is set.
                                                  // But it is so height, that no menu will have so meny entries

    for (int i = 0; i < 5000; i++) menuEntriesData[i] = new MenuEntriesData();


//     menuEntriesData[0].setCaption("Hallo");

    lstMenuEntries = new JList();
    lstMenuEntries.addListSelectionListener(this);
    lstMenuEntries.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    lstMenuEntries.setModel(new DefaultListModel());
    dlmMenuEntries = (DefaultListModel) lstMenuEntries.getModel();
    dlmMenuEntries.add(0, " ");
    lstMenuEntries.setSelectedIndex(0);
    spMenuEntries = new JScrollPane(lstMenuEntries);
    spMenuEntries.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
    spMenuEntries.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
    spMenuEntries.setBounds(15,220,480,250);
    add(spMenuEntries);
    vpMenuEntries = spMenuEntries.getViewport();
    
    JButton btnLoadXML = new JButton("Load XML");
    btnLoadXML.setBounds(310,18,180,23);
    btnLoadXML.setMnemonic('L');
    btnLoadXML.addActionListener(this);
    add(btnLoadXML);
    
    JButton btnSaveXML = new JButton("Save XML");
    btnSaveXML.setBounds(310,50,180,23);
    btnSaveXML.setMnemonic('S');
    btnSaveXML.addActionListener(this);
    add(btnSaveXML);
    
    btnClipboard = new JButton("Copy to Clipboard");
    btnClipboard.setBounds(310,82,180,23);
    btnClipboard.setMnemonic('C');
    btnClipboard.addActionListener(this);
    add(btnClipboard);
 
  }

  private void createButtonImages() {
    Graphics2D graphics;

    /* Left Button Image  */
    graphics = bimgLeftArrow.createGraphics();
    graphics.setColor(Color.black);
    for (int i=4; i>=0; i--) {
      graphics.drawLine(4-i,i,4-i,8-i);   
    }
    for (int i=8; i>=5; i--) {
      graphics.drawLine(i,3,i,5);
    }

    /* Right Button Image */
    graphics = bimgRightArrow.createGraphics();
    graphics.setColor(Color.black);
    for (int i=0; i<=3; i++) {
      graphics.drawLine(i,3,i,5);   
    }
    for (int i=0; i<=4; i++) {
      graphics.drawLine(i+4,i,i+4,8-i);
    }

    /* Up Button Image  */
    graphics = bimgUpArrow.createGraphics();
    graphics.setColor(Color.black);
    for (int i=4; i>=0; i--) {
      graphics.drawLine(i,4-i,8-i,4-i);   
    }
    for (int i=8; i>=5; i--) {
      graphics.drawLine(3,i,5,i);
    }

    /* Down Button Image */
    graphics = bimgDownArrow.createGraphics();
    graphics.setColor(Color.black);
    for (int i=0; i<=3; i++) {
      graphics.drawLine(3,i,5,i);   
    }
    for (int i=0; i<=4; i++) {
      graphics.drawLine(i,i+4,8-i,i+4);
    }

  }
  
  public void paint(Graphics g) {
    super.paint(g);
    //Graphics2D g2 = (Graphics2D) g;
    g.setColor(new Color(255,255,255));
    g.drawRect(11, 177, 490, 0);
    g.setColor(new Color(0,0,0));
    g.drawRect(10, 176, 490, 0);
    g.setColor(new Color(255,255,255));
    g.drawRect(301,11,200,100);
    g.setColor(new Color(0,0,0));
    g.drawRect(300,10,200,100);
  }

  /* main method / starting point */
  public static void main(String[] args) {
    try {
      UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());     
    }catch(Exception e) {} 
    JFrame myFrame = new JFrame("Menu Editor");
    MenuCreator myPanel = new MenuCreator();
    myFrame.add(myPanel);
    myFrame.pack();
    myFrame.setLocationRelativeTo(null);
    myFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    myFrame.setResizable(false);
    myFrame.setVisible(true);
  }

  /* _clicked events */
  public void actionPerformed(ActionEvent ae) {
    Object eventSource = ae.getSource();

    if (eventSource == chbEnabled) {
      chbEnabled_clicked();
    } else if (eventSource == chbVisible) {
      chbVisible_clicked();
    } else if (eventSource == btnLeft) {
      btnLeft_clicked();
    } else if (eventSource == btnRight) {
      btnRight_clicked();
    } else if (eventSource == btnUp) {
      btnUp_clicked();
    } else if (eventSource == btnDown) {
      btnDown_clicked();
    } else if (eventSource == btnNext) {
      btnNext_clicked();
    } else if (eventSource == btnInsert) {
      btnInsert_clicked();
    } else if (eventSource == btnDelete) {
      btnDelete_clicked();
    } else if (eventSource == cbItemtype) {
      cbItemtype_changed();
    } else if (eventSource == chbChecked) {
      chbChecked_changed();
    } else if (eventSource == cbOptionGroup) {
      cbOptionGroup_changed();
    } else if (eventSource == cbShortcut) {
      cbShortcut_changed();
    } else if (eventSource == btnClipboard) {
      btnClipboard_clicked();
    }
  }

  private void chbEnabled_clicked() {
    if ( chbEnabled.getAccessibleContext().getAccessibleValue().getCurrentAccessibleValue().intValue() == 1 ) {
      menuEntriesData[lstMenuEntries.getSelectedIndex()].setEnabled(true);
    } else if ( chbEnabled.getAccessibleContext().getAccessibleValue().getCurrentAccessibleValue().intValue() == 0 ) {
      menuEntriesData[lstMenuEntries.getSelectedIndex()].setEnabled(false);
    }
  }

  private void chbVisible_clicked() {
    if ( chbVisible.getAccessibleContext().getAccessibleValue().getCurrentAccessibleValue().intValue() == 1 ) {
      menuEntriesData[lstMenuEntries.getSelectedIndex()].setVisible(true);
    } else if ( chbVisible.getAccessibleContext().getAccessibleValue().getCurrentAccessibleValue().intValue() == 0 ) {
      menuEntriesData[lstMenuEntries.getSelectedIndex()].setVisible(false);
    }
  }

  private void btnLeft_clicked() {
    int menuItemValue = menuEntriesData[lstMenuEntries.getSelectedIndex()].getMenuItem();
    if (menuItemValue > 0) {
      menuEntriesData[lstMenuEntries.getSelectedIndex()].setMenuItem( menuItemValue-1 );
    }
//    lstMenuEntries.setSelectedIndex( lstMenuEntries.getSelectedIndex() );
    updateTop();
    tfCaption_changed();
  }

  private void btnRight_clicked() {
    int menuItemValue = menuEntriesData[lstMenuEntries.getSelectedIndex()].getMenuItem();
    if (menuItemValue < 7) {
      menuEntriesData[lstMenuEntries.getSelectedIndex()].setMenuItem( menuItemValue+1 );
    }
//    lstMenuEntries.setSelectedIndex( lstMenuEntries.getSelectedIndex() );
    updateTop();
    tfCaption_changed();
  }

  private void btnUp_clicked() {
    int currentSel = lstMenuEntries.getSelectedIndex();
    
    if (currentSel > 0) {
    
      String tmpCaption = menuEntriesData[currentSel].getCaption();
      String tmpName = menuEntriesData[currentSel].getName();
      boolean tmpEnabled = menuEntriesData[currentSel].getEnabled();
      boolean tmpVisible = menuEntriesData[currentSel].getVisible();
      int tmpMenuItem = menuEntriesData[currentSel].getMenuItem();
  
      menuEntriesData[currentSel].setCaption( menuEntriesData[currentSel-1].getCaption() );
      menuEntriesData[currentSel].setName( menuEntriesData[currentSel-1].getName() );
      menuEntriesData[currentSel].setEnabled( menuEntriesData[currentSel-1].getEnabled() );
      menuEntriesData[currentSel].setVisible( menuEntriesData[currentSel-1].getVisible() );
      menuEntriesData[currentSel].setMenuItem( menuEntriesData[currentSel-1].getMenuItem() );

      menuEntriesData[currentSel-1].setCaption( tmpCaption );
      menuEntriesData[currentSel-1].setName( tmpName );
      menuEntriesData[currentSel-1].setEnabled( tmpEnabled );
      menuEntriesData[currentSel-1].setVisible( tmpVisible );
      menuEntriesData[currentSel-1].setMenuItem( tmpMenuItem );
      

      String tmpListEntry = dlmMenuEntries.getElementAt( currentSel ).toString();
      dlmMenuEntries.setElementAt(dlmMenuEntries.getElementAt( currentSel-1 ).toString() , currentSel );
      dlmMenuEntries.setElementAt( tmpListEntry, currentSel-1 );
      lstMenuEntries.setSelectedIndex(currentSel-1);
      lstMenuEntries.ensureIndexIsVisible(lstMenuEntries.getSelectedIndex());
      updateTop();
      tfCaption_changed();
    }
  }

  private void btnDown_clicked() {
    int currentSel = lstMenuEntries.getSelectedIndex();
       
    String tmpCaption = menuEntriesData[currentSel].getCaption();
    String tmpName = menuEntriesData[currentSel].getName();
    boolean tmpEnabled = menuEntriesData[currentSel].getEnabled();
    boolean tmpVisible = menuEntriesData[currentSel].getVisible();
    int tmpMenuItem = menuEntriesData[currentSel].getMenuItem();
 
    menuEntriesData[currentSel].setCaption( menuEntriesData[currentSel+1].getCaption() );
    menuEntriesData[currentSel].setName( menuEntriesData[currentSel+1].getName() );
    menuEntriesData[currentSel].setEnabled( menuEntriesData[currentSel+1].getEnabled() );
    menuEntriesData[currentSel].setVisible( menuEntriesData[currentSel+1].getVisible() );
    menuEntriesData[currentSel].setMenuItem( menuEntriesData[currentSel+1].getMenuItem() );

    menuEntriesData[currentSel+1].setCaption( tmpCaption );
    menuEntriesData[currentSel+1].setName( tmpName );
    menuEntriesData[currentSel+1].setEnabled( tmpEnabled );
    menuEntriesData[currentSel+1].setVisible( tmpVisible );
    menuEntriesData[currentSel+1].setMenuItem( tmpMenuItem );
    

    String tmpListEntry = dlmMenuEntries.getElementAt( currentSel ).toString();
    dlmMenuEntries.setElementAt(dlmMenuEntries.getElementAt( currentSel+1 ).toString() , currentSel );
    dlmMenuEntries.setElementAt( tmpListEntry, currentSel+1 );
    lstMenuEntries.setSelectedIndex(currentSel+1);
    lstMenuEntries.ensureIndexIsVisible(lstMenuEntries.getSelectedIndex());
    updateTop();
    tfCaption_changed();
  }

  private void btnNext_clicked() {
    double vpos;
    if (lstMenuEntries.getSelectedIndex() < dlmMenuEntries.getSize()-1) {
      int menuItemValue = menuEntriesData[lstMenuEntries.getSelectedIndex()].getMenuItem();
      lstMenuEntries.setSelectedIndex( lstMenuEntries.getSelectedIndex()+1 );
      if (lstMenuEntries.getSelectedIndex() >= dlmMenuEntries.getSize()-1) {
        menuEntriesData[lstMenuEntries.getSelectedIndex()].setMenuItem( menuItemValue );
      }
    } else {
      lstMenuEntries.setSelectedIndex(0);
    }
    updateTop();
    lstMenuEntries.ensureIndexIsVisible(lstMenuEntries.getSelectedIndex());
    updateTop();
    tfCaption_changed();
  }

  private void btnInsert_clicked() {
    int currentSel = lstMenuEntries.getSelectedIndex();
    for (int i=dlmMenuEntries.getSize(); i>=currentSel+1; i--) {
      menuEntriesData[i].setCaption(menuEntriesData[i-1].getCaption());
      menuEntriesData[i].setName(menuEntriesData[i-1].getName());
      menuEntriesData[i].setEnabled(menuEntriesData[i-1].getEnabled());
      menuEntriesData[i].setVisible(menuEntriesData[i-1].getVisible());
      menuEntriesData[i].setMenuItem(menuEntriesData[i-1].getMenuItem());
      menuEntriesData[i].setType(menuEntriesData[i-1].getType());
      menuEntriesData[i].setChecked(menuEntriesData[i-1].getChecked());
      menuEntriesData[i].setGroup(menuEntriesData[i-1].getGroup());
      menuEntriesData[i].setShortcut(menuEntriesData[i-1].getShortcut());
    }
    menuEntriesData[currentSel].setCaption("");
    menuEntriesData[currentSel].setName("");
    menuEntriesData[currentSel].setEnabled(true);
    menuEntriesData[currentSel].setVisible(true);
    menuEntriesData[currentSel].setMenuItem(menuEntriesData[currentSel+1].getMenuItem());
    menuEntriesData[currentSel].setType(0);
    menuEntriesData[currentSel].setChecked(false);
    menuEntriesData[currentSel].setGroup(0);
    menuEntriesData[currentSel].setShortcut(0);
 
    dlmMenuEntries.add( currentSel , " ");
    lstMenuEntries.setSelectedIndex(currentSel);
    tfCaption.setText("");
    tfName.setText("");
    chbEnabled.getAccessibleContext().getAccessibleValue().setCurrentAccessibleValue(1);
    chbVisible.getAccessibleContext().getAccessibleValue().setCurrentAccessibleValue(1);
    lstMenuEntries.ensureIndexIsVisible(lstMenuEntries.getSelectedIndex());
    updateTop();
    tfCaption_changed();
  }

  private void btnDelete_clicked() {
    int currentSel = lstMenuEntries.getSelectedIndex();

    if (dlmMenuEntries.getSize() > 1) {
    
      for (int i=currentSel; i<=dlmMenuEntries.getSize()-2; i++) {
        //dlmMenuEntries.setElementAt(menuEntriesData[i+1].getCaption(), i );
        menuEntriesData[i].setCaption(menuEntriesData[i+1].getCaption());
        menuEntriesData[i].setName(menuEntriesData[i+1].getName());
        menuEntriesData[i].setEnabled(menuEntriesData[i+1].getEnabled());
        menuEntriesData[i].setVisible(menuEntriesData[i+1].getVisible());
        menuEntriesData[i].setMenuItem(menuEntriesData[i+1].getMenuItem());
        menuEntriesData[i].setType(menuEntriesData[i+1].getType());
        menuEntriesData[i].setChecked(menuEntriesData[i+1].getChecked());
        menuEntriesData[i].setGroup(menuEntriesData[i+1].getGroup());
        menuEntriesData[i].setShortcut(menuEntriesData[i+1].getShortcut());
      }

      for (int i=dlmMenuEntries.getSize()-1; i<=dlmMenuEntries.getSize()+1; i++) {
        menuEntriesData[i+1].setMenuItem(0);
      }
    
      
      dlmMenuEntries.remove(currentSel);

   
      if (dlmMenuEntries.getSize()-1>=currentSel) {
        lstMenuEntries.setSelectedIndex(currentSel);
      } else {
        lstMenuEntries.setSelectedIndex(currentSel-1);
      }
    }
    lstMenuEntries.ensureIndexIsVisible(lstMenuEntries.getSelectedIndex());
    updateTop();
    tfCaption_changed();
  }

  private void cbItemtype_changed() {
    int currentSel = lstMenuEntries.getSelectedIndex();
    menuEntriesData[currentSel].setType(cbItemtype.getSelectedIndex());
    
    if (menuEntriesData[currentSel].getType() == 0) {
      chbChecked.setVisible(false);
      lblOptionGroup.setVisible(false);
      cbOptionGroup.setVisible(false);
    } else if (menuEntriesData[currentSel].getType() == 1) {
      chbChecked.setVisible(true);
      lblOptionGroup.setVisible(false);
      cbOptionGroup.setVisible(false);
    } else if (menuEntriesData[currentSel].getType() == 2) {
      chbChecked.setVisible(true);
      lblOptionGroup.setVisible(true);
      cbOptionGroup.setVisible(true);
    }
  }
  
  private void chbChecked_changed() {
    int currentSel = lstMenuEntries.getSelectedIndex();
    if (chbChecked.getAccessibleContext().getAccessibleValue().getCurrentAccessibleValue() == 1) {
      menuEntriesData[currentSel].setChecked(true);
    } else {
      menuEntriesData[currentSel].setChecked(false);
    }
  }
  
  private void cbOptionGroup_changed() {
    int currentSel = lstMenuEntries.getSelectedIndex();
    menuEntriesData[currentSel].setGroup(cbOptionGroup.getSelectedIndex());
  }
  
  private void cbShortcut_changed() {
    int currentSel = lstMenuEntries.getSelectedIndex();
    menuEntriesData[currentSel].setShortcut(cbShortcut.getSelectedIndex());
  }
  
  /* _changed events */
  public void caretUpdate(CaretEvent ce) {
    Object eventSource = ce.getSource();

    if (eventSource == tfCaption) {
      tfCaption_changed();
    } else if (eventSource == tfName) {
      tfName_changed();
    }
  }

  private void tfCaption_changed() {
    menuEntriesData[lstMenuEntries.getSelectedIndex()].setCaption(tfCaption.getText());
    if (tfCaption.getText().equals("")) {
      if (menuEntriesData[lstMenuEntries.getSelectedIndex()].getMenuItem() == 0) {
        dlmMenuEntries.setElementAt(" ", lstMenuEntries.getSelectedIndex() );
      } else {
        dlmMenuEntries.setElementAt(menuSpacer(menuEntriesData[lstMenuEntries.getSelectedIndex()].getMenuItem()) + tfCaption.getText(), lstMenuEntries.getSelectedIndex() );
      }
    }  else if (tfCaption.getText().equals("-")) {
      Character a=0x2500;
      String text = "";
      for (int i=0; i<=20; i++) {
        text+=a.toString();
      }
      dlmMenuEntries.setElementAt(menuSpacer(menuEntriesData[lstMenuEntries.getSelectedIndex()].getMenuItem()) + text, lstMenuEntries.getSelectedIndex() );
      if (dlmMenuEntries.getSize()-1 == lstMenuEntries.getSelectedIndex()) {
        dlmMenuEntries.add(lstMenuEntries.getSelectedIndex()+1 , " ");
      }
    } else {
      if (tfCaption.getText().length() > 5 && tfCaption.getText().substring(0,6).equalsIgnoreCase("<html>")) {
        dlmMenuEntries.setElementAt("<html>" + menuSpacer(menuEntriesData[lstMenuEntries.getSelectedIndex()].getMenuItem()) + tfCaption.getText().substring(6,tfCaption.getText().length()), lstMenuEntries.getSelectedIndex() );
      } else {
        dlmMenuEntries.setElementAt(menuSpacer(menuEntriesData[lstMenuEntries.getSelectedIndex()].getMenuItem()) + tfCaption.getText(), lstMenuEntries.getSelectedIndex() );
      }
      if (dlmMenuEntries.getSize()-1 == lstMenuEntries.getSelectedIndex()) {
        dlmMenuEntries.add(lstMenuEntries.getSelectedIndex()+1 , " ");
      }
    }
  }

  private void tfName_changed() {
    menuEntriesData[lstMenuEntries.getSelectedIndex()].setName(tfName.getText());
  }


  public void valueChanged(ListSelectionEvent lse) {
    Object eventSource = lse.getSource();

    if (eventSource == lstMenuEntries) {
      lstMenuEntries_changed();
    }
  }

  private void lstMenuEntries_changed() {
    updateTop();
  }


  /* return spacers of the number of MenuItems (deepness of the menu) */
  private String menuSpacer(int number) {
    String tmpString ="";
    Character a = 0xb7;
    for (int i=1; i<=number; i++) {
      tmpString += a.toString() + a.toString() + a.toString() + a.toString();
    }
    return tmpString;
  }
  
  /* if you use one of the buttons or the list control, this updates the thing on the top */
  private void updateTop() {
    int currentSel = lstMenuEntries.getSelectedIndex();
    if (currentSel > -1) {
      if (menuEntriesData[currentSel].getCaption()==null)
        menuEntriesData[currentSel].setCaption("");
      if (menuEntriesData[currentSel].getName()==null)
        menuEntriesData[currentSel].setName("");
    
      tfCaption.setText( menuEntriesData[currentSel].getCaption() );
      tfCaption.setCaretPosition(0);
      tfName.setText( menuEntriesData[currentSel].getName() );
      tfName.setCaretPosition(0);

      if (menuEntriesData[currentSel].getEnabled() == true) {
        chbEnabled.getAccessibleContext().getAccessibleValue().setCurrentAccessibleValue(1);
      } else {
        chbEnabled.getAccessibleContext().getAccessibleValue().setCurrentAccessibleValue(0);
      }

      if (menuEntriesData[currentSel].getVisible() == true) {
        chbVisible.getAccessibleContext().getAccessibleValue().setCurrentAccessibleValue(1);
      } else {
        chbVisible.getAccessibleContext().getAccessibleValue().setCurrentAccessibleValue(0);
      }

      if (menuEntriesData[currentSel].getChecked() == true) {
        chbChecked.getAccessibleContext().getAccessibleValue().setCurrentAccessibleValue(1);
      } else {
        chbChecked.getAccessibleContext().getAccessibleValue().setCurrentAccessibleValue(0);
      }
      
      cbItemtype.setSelectedIndex(menuEntriesData[currentSel].getType());
      if (menuEntriesData[currentSel].getType() == 0) {
        chbChecked.setVisible(false);
        lblOptionGroup.setVisible(false);
        cbOptionGroup.setVisible(false);
      } else if (menuEntriesData[currentSel].getType() == 1) {
        chbChecked.setVisible(true);
        lblOptionGroup.setVisible(false);
        cbOptionGroup.setVisible(false);
      } else if (menuEntriesData[currentSel].getType() == 2) {
        chbChecked.setVisible(true);
        lblOptionGroup.setVisible(true);
        cbOptionGroup.setVisible(true);
      }

      cbOptionGroup.setSelectedIndex(menuEntriesData[currentSel].getGroup());
      
      cbShortcut.setSelectedIndex(menuEntriesData[currentSel].getShortcut());
    }
  }
  
  
  
  
  ///////////               \\\\\\\\\\\\\
  //////////  NON-GUI PART   \\\\\\\\\\\\
  /////////                   \\\\\\\\\\\
  
  
  private void  writeError(int lineNr, String text) {
    lstMenuEntries.setSelectedIndex(lineNr);
    lstMenuEntries.ensureIndexIsVisible(lineNr);
    JOptionPane.showMessageDialog(this, text, "ERROR!", JOptionPane.ERROR_MESSAGE);
    
    // empty clipboard
    java.awt.datatransfer.StringSelection stringSelection = new java.awt.datatransfer.StringSelection("");
    java.awt.datatransfer.Clipboard clpbrd = Toolkit.getDefaultToolkit().getSystemClipboard();
    clpbrd.setContents(stringSelection, null);
  }
  
  private String getCaptionWay(int menuitem, String[] hlc) {
    String str="";
    for (int i=0; i<=menuitem; i++) {
      if (i==0) {
        str += "\"" + hlc[i] + "\"";
      } else {
        str += " ==> \"" + hlc[i] + "\"";
      }
    }
    return str;
  }
  
  private String getKeystrokeString(int sc) {
    // sc = shortcut
    String str="Keystroke.getKeyStroke(";
    if (sc > 0 && sc <= 26) {
      str += (KeyEvent.VK_A - 1 + sc) + ",  " + Event.CTRL_MASK;
    } else if (sc > 26 &&  sc <= 38 ) {
      str += (KeyEvent.VK_F1 - 27 + sc) + ", 0";
    } else if (sc > 38 &&  sc <= 50 ) {
      str += (KeyEvent.VK_F1 - 39 + sc) + ", " + Event.CTRL_MASK;
    } else if (sc > 50 &&  sc <= 62 ) {
      str += (KeyEvent.VK_F1 - 51 + sc) + ", " + (Event.CTRL_MASK | Event.SHIFT_MASK);
    } else if (sc == 63) {
      str += KeyEvent.VK_INSERT + ", " + Event.CTRL_MASK;
    } else if (sc == 64) {
      str += KeyEvent.VK_INSERT + ", " + Event.SHIFT_MASK;
    } else if (sc == 65) {
      str += KeyEvent.VK_DELETE + ", 0";
    } else if (sc == 66) {
      str += KeyEvent.VK_DELETE + ", " + Event.SHIFT_MASK;
    } else if (sc == 67) {
      str += KeyEvent.VK_BACK_SPACE  + ", " + Event.ALT_MASK;
    }
    str += ")";
    return str;
  }
  
  private void btnClipboard_clicked() {
  
  
    int endArray=0;
    
    // *** end array check ***
    
    for (int i=dlmMenuEntries.getSize()-1; i>=0; i--) {
       if (menuEntriesData[i].getName()!=null && (!menuEntriesData[i].getName().equals("") || !menuEntriesData[i].getCaption().equals("")) ) {
         endArray=i;
         break;
       }
    }

    
    //  *** error-check ***
    
    // check, if all entries have a name
    for (int i=0; i<=endArray; i++) {
      if (menuEntriesData[i].getName().equals("")) {
        writeError(i, "Menu control must have a name");
        return;
      }
    }

    // check, if all names are unique
    {
      String tmpArray[] = new String[5000];
      int tmpVal = 0;
      
      tmpArray[0]=menuEntriesData[0].getName();
      for (int i=1; i<=endArray; i++) {
        if (!menuEntriesData[0].getCaption().equals("-")) {
          for (int k=1; k<=tmpVal; k++) {
            if (menuEntriesData[i].getName().equalsIgnoreCase(tmpArray[k])) {
              writeError(i, "Menu control must have a unique name");
              return;
            }
          }
          tmpVal++;
          tmpArray[tmpVal]=menuEntriesData[i].getName();
        }
      }
    }

    // check, if name is allowed
    {
      String fb = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";  // first byte
      String ob = fb + "1234567890_"; // other byte
      String chr= ""; // a character
      for (int i=0; i<=endArray; i++) {
        for (int k=0; k<=menuEntriesData[i].getName().length()-1; k++) {
          int v;
          chr=menuEntriesData[i].getName().substring(k, k+1);
          if (k==0) {
            v = fb.indexOf(chr);
          } else {
            v = ob.indexOf(chr);
          }
          if (v<0) {
            writeError(i, "Not a legal object name: '" + menuEntriesData[i].getName() + "'");
            return;
          }
        }
      }
    }

    // hirarchical check
    {
      int val=0;
      int cur=0;
      for (int i=0; i<=endArray; i++) {
        cur=menuEntriesData[i].getMenuItem();
        if (cur > val+1) {
          writeError(i, "Menu item skipped a level");
          return;
        }
        val = cur;
      }
    }
  
    // check, if checkbox or optionbutton is on the top
    for (int i=0; i<=endArray; i++) {
      if (menuEntriesData[i].getMenuItem()==0) {
        if (menuEntriesData[i].getType()!=0) {
          writeError(i, "At the top of the menu-tree are CheckBox\nand OptionButton not allowed!");
          return;
        }
      }
    }
  
    // create leaf-array and look, if separators, checkboxes and optionbuttons are leafs
    boolean leafs[] = new boolean[5000];
    int last=0;
    for (int i=endArray; i>=0; i--) {
      if (menuEntriesData[i].getMenuItem() < last) {
        leafs[i] = false;
        if (menuEntriesData[i].getType()!=0) {
          writeError(i, "CheckBox and OptionButton must be a leaf of the tree!");
          return;
        } else if (menuEntriesData[i].getCaption().equals("-")) {
          writeError(i, "Separator must be a leaf of the tree!");
          return;
        }
      } else {
        leafs[i] = true;
      }
      last=menuEntriesData[i].getMenuItem();
    }
    
    
    //  *** generate the Jabaco-code ***
  

    String text = "";  // the complete text, which will be copied to the clipboard
    String dt = "";    // Dim text; (the global variables at the top)
    String flt = "";   // Form_Load() text
    String fmct = "";  // Form_MenuClick() text
    String st = "";    // Sub text (the subs, which are called by the menu entries)

    String hln[] = new String[10]  ; // higher level name
    String hlc[] = new String[10]  ; // higher level caption   
    
    boolean gn[] = new boolean[10]; // group number ; saves which groups exist.
    for (int i=0; i<=9; i++) {
      gn[i]=false;
    }
    
    for (int i=0; i<=endArray; i++) {
      MenuEntriesData med = menuEntriesData[i];
      boolean leaf = leafs[i];
      
      if (med.getMenuItem()==0) {
        dt += "Dim byMenuEditor_" + med.getName() + " As VBMenuGroup\n";
        flt += "   byMenuEditor_" + med.getName() + " = Me.MenuBar.createMenuGroup(Null, \"" + med.getName() + "\", \"" + med.getCaption() + "\", " + med.getVisibleAsString() + ", Nothing)\n" +
               "   byMenuEditor_" + med.getName() + ".Parent.setEnabled(" + med.getEnabledAsString() + ")\n";
        hln[0] = med.getName();
        hlc[0] = med.getCaption();
      } else {
        if (med.getCaption().equals("-")) {
          flt += "   Call Me.MenuBar.addSeperator(byMenuEditor_" + hln[med.getMenuItem()-1] + ")\n";
        } else {
          if (med.getType()==0) {
            if (leaf) {
              dt += "Dim byMenuEditor_" + med.getName() + " As VBMenuItem\n";
              flt += "   byMenuEditor_" + med.getName() + " = Me.MenuBar.createMenu(byMenuEditor_" + hln[med.getMenuItem()-1] + ", \"" + med.getName() + "\", \"" + med.getCaption() + "\", " + med.getVisibleAsString() + ", Nothing)\n" +
                     "   byMenuEditor_" + med.getName() + ".Parent.setEnabled(" + med.getEnabledAsString() + ")\n";
              hln[med.getMenuItem()] = med.getName();
              hlc[med.getMenuItem()] = med.getCaption();
              if (med.getShortcut() > 0) {
                flt += "   byMenuEditor_" + med.getName() + ".Parent.setAccelerator(" + getKeystrokeString(med.getShortcut()) + ")\n";
              }
              fmct += "      Case \"" + med.getName() + "\"\n         " + med.getName() + "_Click()\n";
              st += "Public Sub " + med.getName() + "_Click()\n   ' " + getCaptionWay(med.getMenuItem(), hlc) + "\nEnd Sub\n\n";
            } else {
              dt += "Dim byMenuEditor_" + med.getName() + " As VBMenuGroup\n";
              flt += "   byMenuEditor_" + med.getName() + " = Me.MenuBar.createMenuGroup(byMenuEditor_" + hln[med.getMenuItem()-1] + ", \"" + med.getName() + "\", \"" + med.getCaption() + "\", " + med.getVisibleAsString() + ", Nothing)\n" +
                     "   byMenuEditor_" + med.getName() + ".Parent.setEnabled(" + med.getEnabledAsString() + ")\n";
              hln[med.getMenuItem()] = med.getName();
              hlc[med.getMenuItem()] = med.getCaption();
              if (med.getShortcut() > 0) {
                flt += "   byMenuEditor_" + med.getName() + ".Parent.setAccelerator(" + getKeystrokeString(med.getShortcut()) + ")\n";
              }
            }
          } else if (med.getType()==1) {
            dt += "Dim byMenuEditor_" + med.getName() + " As VBCheckBoxMenuItem\n";
            flt += "   byMenuEditor_" + med.getName() + " = Me.MenuBar.createCheckBoxMenu(byMenuEditor_" + hln[med.getMenuItem()-1] + ", \"" + med.getName() + "\", \"" + med.getCaption() + "\", " + med.getCheckedAsString() + ", " + med.getVisibleAsString() + ", Nothing)\n" +
                   "   byMenuEditor_" + med.getName() + ".Parent.setEnabled(" + med.getEnabledAsString() + ")\n";
            hln[med.getMenuItem()] = med.getName();
            hlc[med.getMenuItem()] = med.getCaption();
            if (med.getShortcut() > 0) {
              flt += "   byMenuEditor_" + med.getName() + ".Parent.setAccelerator(" + getKeystrokeString(med.getShortcut()) + ")\n";
            }
            fmct += "      Case \"" + med.getName() + "\"\n         " + med.getName() + "_Click()\n";
            st += "Public Sub " + med.getName() + "_Click()\n   ' " + getCaptionWay(med.getMenuItem(), hlc) + "\nEnd Sub\n\n";          
          } else if (med.getType()==2) {
            dt += "Dim byMenuEditor_" + med.getName() + " As VBRadioButtonMenuItem\n";
            if (gn[med.getGroup()]) {
              flt += "   byMenuEditor_" + med.getName() + " = Me.MenuBar.createRadioButtonMenu(byMenuEditor_" + hln[med.getMenuItem()-1] + ", \"" + med.getName() + "\", \"" + med.getCaption() + "\", " + med.getCheckedAsString() + ", byMenuEditorButtonGroup_" + med.getGroup() +  ", " + med.getVisibleAsString() + ", Nothing)\n" +
                     "   byMenuEditor_" + med.getName() + ".Parent.setEnabled(" + med.getEnabledAsString() + ")\n";
            } else {
              flt += "   Dim byMenuEditorButtonGroup_" + med.getGroup() + " As New javax#swing#ButtonGroup\n" +
                     "   byMenuEditor_" + med.getName() + " = Me.MenuBar.createRadioButtonMenu(byMenuEditor_" + hln[med.getMenuItem()-1] + ", \"" + med.getName() + "\", \"" + med.getCaption() + "\", " + med.getCheckedAsString() + ", byMenuEditorButtonGroup_" + med.getGroup() +  ", " + med.getVisibleAsString() + ", Nothing)\n" +
                     "   byMenuEditor_" + med.getName() + ".Parent.setEnabled(" + med.getEnabledAsString() + ")\n";
              gn[med.getGroup()] = true;
            }            
            hln[med.getMenuItem()] = med.getName();
            hlc[med.getMenuItem()] = med.getCaption();
            if (med.getShortcut() > 0) {
              flt += "   byMenuEditor_" + med.getName() + ".Parent.setAccelerator(" + getKeystrokeString(med.getShortcut()) + ")\n";
            }
            fmct += "      Case \"" + med.getName() + "\"\n         " + med.getName() + "_Click()\n";
            st += "Public Sub " + med.getName() + "_Click()\n   ' " + getCaptionWay(med.getMenuItem(), hlc) + "\nEnd Sub\n\n";
          }
        }
      } 
  

  
  
   }

    
    
    flt = "Public Sub Form_Load()\n" + flt + "\nEnd Sub";
    fmct = "Public Sub Form_MenuClick(MenuItem As VB#IMenuItem)\n   Select Case MenuItem.ControlID\n" + fmct + "      Case Else\n         Debug.Print(\"*** An error happens ***\")\n   End Select\nEnd Sub";
    text = dt + "\n\n" + flt + "\n\n" + fmct + "\n\n" + st + "\n";
    //System.out.println(text);
    java.awt.datatransfer.StringSelection stringSelection = new java.awt.datatransfer.StringSelection(text);
    java.awt.datatransfer.Clipboard clpbrd = Toolkit.getDefaultToolkit().getSystemClipboard();
    clpbrd.setContents(stringSelection, null);
  }
  
}
